import os
import json


def begin():
    notebook = {
                "cells": [
                {
                "cell_type": "markdown",
                "metadata": {},
                "source": [
                    "==SOLUTION==\n",
                    "\n",
                    ">This notebook is meant for curriculum development. \n",
                    "\n",
                    "## Instructions:\n",
                    "\n",
                    "### Overview\n",
                    "\n",
                    "When you have completed writing your lesson, run `ds -create` from the root of this lesson folder. This will create an `index.ipynb` notebook containing the student facing content in the root directory of this lesson. An `index.ipynb` file will be added to the `.solution_files` directory containing all solution materials. \n",
                    "\n",
                    "## How to use this file\n",
                    "\n",
                    "**Solution Tags:**\n",
                    "\n",
                    "Markdown cells: \n",
                    "- `==SOLUTION==`\n",
                    "\n",
                    "Code cells: \n",
                    "- `#__SOLUTION__`\n",
                    "\n",
                    "\n",
                    "#### Create Solution Cells:\n",
                    "- To designate a cell as a solution, place the solution tag on its own line within the cell. This cell is an example of a \"solution markdown cell\".\n",
                    "\n",
                    "- Below is an example of a solution code cell."
                ]
                },
                {
                "cell_type": "code",
                "execution_count": 0,
                "metadata": {},
                "outputs": [],
                "source": [
                    "#__SOLUTION__\n",
                    "\n",
                    "\"\"\"\n",
                    "This is a solution cell and \n",
                    "will not be copied \n",
                    "to the lesson index.ipynb file\n",
                    "\"\"\""
                ]
                }
                ],
                "metadata": {},
                "nbformat": 4,
                "nbformat_minor": 2
                }


    notebook_file = open('curriculum.ipynb', 'w')
    json.dump(notebook, notebook_file)
    notebook_file.close()

    if not os.path.isdir('data'):
        os.mkdir('data')






import json
import os

class GenerateCurriculum:
    """
    Splits a jupyter notebook into lesson and solution notebooks.
    =============================================================
            This class looks for `#__SOLUTION__` in code cells
            and `===SOLUTION===` in markdown cells. If a solution tag
            is found in a code cell, the cell is added to the solution
            notebook and withheld from the lesson notebook.
            If a solution tag is found in a markdown cell, the cell
            is added to the solution notebook and is replaced with
            `*YOUR ANSWER HERE*` in the lesson notebook.
    """
    def __init__(self):
        self.cells = []
        solution_path = os.path.join('.solution_files', 'index.ipynb')
        self.notebook = {
                            "cells": [],
                            "metadata": {},
                            "nbformat": 4,
                            "nbformat_minor": 2
                            }

        

    def get_solution_cells(self):
        with open(solution_path, 'r') as file:
            data = json.load(file)
        
        for idx in range(len(data['cells'])):
            data['cells'][idx]['source'] = ['#==SOLUTION== \n'] + data['cells'][idx]['source']

        self.cells += data['cells']

    def get_lesson_cells(self):
        with open('index.ipynb', 'r') as file:
            data = json.load(file)

        self.cells += data['cells']

    def sort_cells(self):
        cells = []
        for cell in self.cells:
            if 'index' in cell['metadata']:
                if cell['metadata']['index'] != 'Placeholder':
                    cells.append(cell)
            else:
                continue
        self.notebook['cells'] = sorted(cells, key=lambda x: x['metadata']['index'])
        


    def create_curriculum_notebook(self):
        """
        Write a notebook to file.

        """

        file = open(f"curriculum.ipynb", "w")
        file.write(json.dumps(self.notebook))
        file.close()

    def main(self, filename="curriculum.ipynb"):
        """
        Main function for running the splitting process.
        """
        self.get_lesson_cells()
        self.get_solution_cells()
        self.sort_cells()
        self.create_curriculum_notebook()


from .ShareNotebook import ShareNotebook
from .SplitNotebook import SplitNotebook
from .Color import Color
from .begin import begin
from .GenerateCurriculum import GenerateCurriculum
from .readme import generate_readme


import os
import shutil
import nbformat
from nbconvert import MarkdownExporter
from nbconvert.writers import FilesWriter
from traitlets.config import Config
    
def generate_readme(notebook_path, dir_path, filename):
    """
    Saves a jupyter notebook as a README.md file using nbformat and nbconvert

    Inputs:
    1. The path for a jupyter notebook to be converted into markdown. str.
    2. The directory path for saving the README.md file. str.
    3. The name of the file without the `.filetype` (README instead of README.md). str.

    Returns:
    None
    """
    output_path = os.path.join(dir_path, 'README.md')
    index_files = os.path.join(dir_path, 'index_files')
    input_path = os.path.join(index_files, filename + '.md')
    notebook = nbformat.read(notebook_path, nbformat.NO_CONVERT)
    mark_exporter = MarkdownExporter()
    (output, resources) = mark_exporter.from_notebook_node(notebook)


    if not os.path.isdir(index_files):
        os.mkdir(index_files)
    if os.path.isfile(output_path):
        os.remove(output_path)
    if os.path.isfile(input_path):
        os.remove(input_path)

    c = Config()
    c.FilesWriter.build_directory = index_files
    fw = FilesWriter(config=c)
    fw.write(output, resources, notebook_name=filename)
    shutil.move(input_path, output_path)


import os
import subprocess
from IPython.display import clear_output


def run_bash_command(command):
    process = subprocess.Popen(command.split(), stdout=subprocess.PIPE)
    process.communicate()

def spark_setup(drive=True):
    print('Setting up PySpark.')
    run_bash_command('pip install tqdm -q')
    from tqdm import tqdm
    pbar = tqdm(total=7)
    clear_output()
    print('Updating system configurations.')
    pbar.update(1)
    if os.name == 'posix' and not drive:
        run_bash_command('brew tap AdoptOpenJDK/openjdk')
        run_bash_command('brew install --cask adoptopenjdk8')
    else:
        run_bash_command('apt -qq update ')
        run_bash_command(f'apt-get install openjdk-8-jdk-headless -qq')
    clear_output()
    print('Downloading spark-2.4.8-bin-hadoop2.')
    pbar.update(1)
    run_bash_command('wget -q https://downloads.apache.org/spark/spark-2.4.8/spark-2.4.8-bin-hadoop2.7.tgz')
    clear_output()
    print('Unzipping file')
    pbar.update(1)
    run_bash_command('tar xf spark-2.4.8-bin-hadoop2.7.tgz')
    clear_output()
    print('Installing findspark')
    pbar.update(1)
    run_bash_command('pip install -q findspark')
    clear_output()
    print('Setting environment paths.')
    pbar.update(1)

  
    os.environ["JAVA_HOME"] = "/usr/lib/jvm/java-1.8.0-openjdk-amd64"
    os.environ["SPARK_HOME"] = os.getcwd() + '/spark-2.4.8-bin-hadoop2.7'
    clear_output()
    print('Running findspark')
    pbar.update(1)

    import findspark
    findspark.init()
    clear_output()
    print('Complete!')
    pbar.update(1)


import os

def get_nbgrader_directory():
    """
    Helper function for finding the top level directory of an nbgrader course.

    This function searches for the word 'source' in the absolute path of the current working directory, and uses the
    placement of the source folder to find the top level of the nbgrader course

    Returns: str. directory path.
    """

    # Get current working directory
    path = os.getcwd()
    # Split and reverse the cwd path
    split = path.split(os.sep)[::-1]
    if 'source' not in split:
        raise ValueError('ds -generate must be run from an nbgrader assignment folder within the source/ directory.')
    # Move up a directory for each directory beneath and equal to the source directory
    top_directory = ''
    for idx in range(len(split)):
        top_directory = os.path.join(top_directory, '..')
        if split[idx] == 'source':
            break

    return top_directory


import pyperclip
import argparse

class ShareNotebook:

    def get_file_path(self, url):
        """
        Pull out the organization, repository name, branch, and file path
        from a github url.
        """
        org, repo = url.split('github.com/')[1].split('/')[:2]
        paths = url.split('blob/')[1].split('/')
        branch = paths[0]
        file_path = '/'.join(paths[1:])
        return org, repo, branch, file_path

    def get_assignment_url(self, org, repo, branch, file_path):
        """
        org - The name of a github organization.
        repo - The name of a github repository.
        branch - The name of a github repository branch.
        file_path - The path pointing to a jupyter notebook in a github repository.

        Returns: An illumidesk link that will clone the notebook onto your personal
                server and open the notebook.
        """
        template = """https://flatiron.illumidesk.com/user/x/git-pull?repo=https%3A%2F%2Fgithub.com%2F{}%2F{}&branch={}&subpath={}"""
        return template.format(org, repo, branch, file_path)

    def main(self, url):
        """
        Main function for creating the illumidesk link.
        The link is added to your clipboard.
        """
        org, repo, branch, file_path = self.get_file_path(url)
        link = self.get_assignment_url(org, repo, branch, file_path)
        pyperclip.copy(link)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Create an illumidesk link for a github hosted notebook.')
    parser.add_argument('url', action="store")
    args = parser.parse_args()
    url = args.url
    share = ShareNotebook()
    share.main(url)
    print()
    print('Url has been copied to your clipboard!')
    print()


import json
import os


class SplitNotebook:
    """
    Splits a jupyter notebook into lesson and solution notebooks.
    =============================================================
            This class looks for `#__SOLUTION__` in code cells
            and `===SOLUTION===` in markdown cells. If a solution tag
            is found in a code cell, the cell is added to the solution
            notebook and withheld from the lesson notebook.
            If a solution tag is found in a markdown cell, the cell
            is added to the solution notebook and is replaced with
            `*YOUR ANSWER HERE*` in the lesson notebook.
    """
    def __init__(self, filename='index.ipynb', dir=False):
        self.solution_tags = ["__SOLUTION__", f"#__SOLUTION__", 
                            "==SOLUTION==", f"#==SOLUTION=="]
        self.data = self.get_notebook_json(filename=filename)
        self.dir = dir

    def get_notebook_json(self, filename="index.ipynb"):
        """
        Import raw json data for a source notebook
        """
        with open(filename, 'r') as f:
            data = json.load(f)
        return data
        
    def split_notebook(self):
        """
        Loops over each cell in a notebook
        and appends all solution and lesson cells to the
        `solution_cells` and `lesson_cells` attributes.
        """
        count = 0
        lesson_cells = []
        solution_cells = []
        for cell in self.data['cells']:
            cell_type = cell['cell_type']
            solution, lines = self._parse_cell(cell)
            cell["source"] = lines
            cell['metadata']['index'] = count
            placeholder = dict(cell)
            placeholder['metadata'] = dict(cell['metadata'])
            placeholder['metadata']['index'] = 'Placeholder'
            
            if cell_type == "markdown":

                if solution:
                    solution_cells.append(cell)
                    placeholder.update({"source": ['*YOUR ANSWER HERE*']})
                    lesson_cells.append(placeholder)
                else:
                    lesson_cells.append(cell)
                    if self.dir:
                        solution_cells.append(placeholder)
            else:
                if solution:
                    solution_cells.append(cell)

                else:
                    lesson_cells.append(cell)
                    if self.dir:
                        solution_cells.append(placeholder) 
            count += 1 
        return lesson_cells, solution_cells 

    def _parse_cell(self, cell):
        """
        Loops over every line in a cell and 
        searches for a solution tag. 
        
        Returns a boolean indicating if a solution tag was found
        in the cel and a list of all lines that did not contain a solution tag. 

        """
        is_solution = False
        lines = []
        for line in cell["source"]:
            found = False
            for tag in self.solution_tags:
                if tag in line.strip().split(" "):
                    is_solution = True
                    found = True
            if not found:
                lines.append(line)
        return is_solution, lines

    def write_notebook(self, name, cells):
        """
        Write a notebook to file.

        name - The name of the notebook file (excluding .ipynb)
        cells - The cells the notebook will contain.
        """
        notebook = dict(self.data)
        notebook.update({"cells": cells})
        file = open(f"{name}.ipynb", "w")
        file.write(json.dumps(notebook))
        file.close()


import sys
from git import Repo, GitCommandError

class GitModel(Repo):

    def __init__(self):
        Repo.__init__(self, '.')
        self.msg = self.get_commit_message()

    def get_commit_message(self):
        # get commit message from repo or custom flag
        sys_args = list(sys.argv)
        i = sys_args.index('-m') if '-m' in sys_args else None
        return sys_args[i + 1] if i else self.head.commit.message

    def add_and_commit(self, commit_msg=None):
        self.git.add(".")
        try:
            self.git.commit("-m", commit_msg if commit_msg else self.msg)
        except GitCommandError:
            print("nothing to commit, working tree clean")

    def merge_solution(self):
        self.git.checkout('solution')
        solution = self.active_branch
        master = self.branches['master']
        base = self.merge_base(solution, master)
        self.index.merge_tree(master, base=base)
        self.index.commit('Merge master into solution.', parent_commits=(solution.commit, master.commit))


import os
import types
import pickle
from IPython.display import display, Markdown

from .BaseTest import BaseTest
from .TestGenerator import TestGenerator


class Tests(BaseTest):
    """
    Controller object for saving and running tests.
    """
    def __init__(self):
        # If a folder for solution objects has not been created
        # it is created here
        if not os.path.isdir('.solution_files'):
            os.mkdir('.solution_files')
        self.test_dir = os.path.join('.solution_files', '.test_obj')
        if not os.path.isdir(self.test_dir):
            os.mkdir(self.test_dir)
            
    def save(self, callable, name, assertion=True):
        """
        Saves a callable function or class object to disk.

        Arguments
        ---------

        callable:      A function or class object.
        name:          String. The name of the test. This argument will be used to run the 
                       the test, and should be representative of what the test is check.

        assertion:     Boolean. Default: True. 
                       When passing in a function object, if assertion=True
                       The function should either return True or False
                       If you would like to return a custom message for the test
                       assertion should be set to False.

        Returns
        -------
        None
        """
        # To run saved callable objects, the objects must be serialized
        # and saved to json. 
        # The json is structured as follows:
              # {'assertion': <boolean>,
              #  'kind': <string either "function" or "class">,
              #  'functions': <function object if kind="string"
              #                 else {"name of class method": function object}}
        data = {}
        data['assertion'] = assertion
        # source: https://stackoverflow.com/a/29250620
        if isinstance(callable, types.FunctionType):
            data['kind'] = 'function'
            callable_ = pickle.dumps(callable.__code__)
            data['functions'] = callable_
        elif isinstance(callable, type):
            data['kind'] = 'class' 
            data['functions'] = {}
            for item in callable.__dict__:
                if isinstance(callable.__dict__[item], types.FunctionType):
                    serialized = pickle.dumps(callable.__dict__[item].__code__)
                    data['functions'][item] = serialized
        path = os.path.join(self.test_dir, name.strip().lower() + '.pkl')
        file = open(path, 'wb')
        pickle.dump(data, file)
        file.close()

    def run(self, name, *args):
        """
        Runs a saved test. 

        Arguments
        ---------
        name:       The name of the test. This is the string that was used
                    to save the test object.

        *args:      This is typically the answer that is being tested. Multiple
                    arguments are allowed.

        Returns
        -------
        A print out for each test.
        """
        path = os.path.join(self.test_dir, name.strip().lower() + '.pkl')
        file = open(path, 'rb')
        data = pickle.load(file)
        file.close()
        
        if data['kind'] == 'function' and data['assertion']:
            function = types.FunctionType(pickle.loads(data['functions']), globals())
            self.print_results(name, function, *args)
            
        elif data['kind'] == 'function':
            function = types.FunctionType(pickle.loads(data['functions']), globals())
            output = function(*args)
            markdown = Markdown(f"""**{name}:** {output}""")
            display(markdown)
        
        elif data['kind'] == 'class':
            generator = TestGenerator(data, *args)
            generator.main()


import pickle
import types

from .BaseTest import BaseTest

class TestGenerator(BaseTest):
    """
    If a saved test is a class, TestGenerator reconstructs the class from
    serialized json. This generator only reconstructs the methods of the original class.
    """
    def __init__(self, data, *args):
        """
        Adds all saved methods from the test object as attributes, and runs 
        the original objects __init__ method.
        """
        for function in data['functions']:
            serialized = data['functions'][function]
            loaded_function = types.FunctionType(pickle.loads(serialized), globals())
            if function == '__init__':
                loaded_function(self, *args)
            else:
                setattr(self, function, loaded_function)

    def main(self):
        """
        Loops over attributes and runs methods that begin with the word `test`.
        """
        for item in self.__dir__():
            if 'test' in item:
                function = self.__getattribute__(item)
                if 'output' in function.__code__.co_varnames:
                    self.print_results(item, function,
                                      main=True, output=True)
                else:
                    self.print_results(item, function,
                                       main=True, output=False)


from setuptools import setup, find_packages

setup(name='dscreate',
      version='0.1.85.34',
      description='Flatiron Iron School Data Science Tools',
      url='http://github.com/learn-co-curriculum/dscreate',
      author='Joél Collins',
      author_email='joelsewhere@gmail.com',
      license='MIT',
      packages=find_packages(),
      install_requires=['argparse', 'pyperclip', 'traitlets', 
                        'nbgrader', 'nbformat', 'nbconvert', 'GitPython'],
      scripts=['bin/ds'],
      zip_safe=False)


"""
Base class for tests. Primarily provides Markdown Outputs.
"""

from IPython.display import display, Markdown

class BaseTest:
    
    def printout(self, outcome, test_name):
        if outcome:
            markdown = Markdown(f"""**{test_name}:** ✅""")
        else:
            markdown = Markdown(f"""**{test_name}:** ❌""")
        display(markdown)
            
    def print_results(self, key, test, *args, main=False, output=False):
        if main and output:
            result = test(self, True)
        elif main:
            result = test(self)
        else:
            result = test(*args)
        if output:
            markdown = Markdown(f"**{key}:** {result}")
            display(markdown)
        else:
            self.printout(result, key)